proc blastAlign_init {} {
    #Creating menu buttons
    .menubar.module.menu add command -label "Blast alignment" -command "blastAlign"
}

proc blastAlign_remove {} {
    #Remove menu buttons
    .menubar.module.menu delete "Blast alignment"
}

proc blastAlign {} {
    global env
    global resWinNum
    
    set resWinNum 0
    
    if {$env(dbName) ne " "} {
        set ncbi_blast_expectCheck 1
        set ncbi_blast_evalue "1e-003"
        set ncbi_blast_hit 5
        set ncbi_blast_threads 5
        
        #  Configure the main window
        set t .localAlign
        catch {blastAlign_resClose; destroy $t}
        toplevel $t
        wm title $t "Blast alignment - $env(dbName)"
        wm minsize $t 250 170
        
        variable seq1
        variable seq2
        
        frame $t.program
        label $t.program.label -text "Program:"
        ComboBox $t.program.cb -width 10 -hottrack 1 -background white -text "blastn" -values {blastn blastp}     
        frame $t.seq1
        frame $t.seq1.head
        label $t.seq1.head.label -text "Sequence 1"
        button $t.seq1.head.clear -text "Clear" -command "$t.seq1.panel delete 1.0 end"
        text $t.seq1.panel -height 5 -background white
        frame $t.seq2
        frame $t.seq2.head
        label $t.seq2.head.label -text "Sequence 2"
        button $t.seq2.head.clear -text "Clear" -command "$t.seq2.panel delete 1.0 end"
        text $t.seq2.panel -height 5 -background white
        
        pack $t.program.label $t.program.cb -side left -anchor w
        pack $t.seq1.head.label -side left; pack $t.seq1.head.clear -side right
        pack $t.seq2.head.label -side left; pack $t.seq2.head.clear -side right
        pack $t.seq1.head -side top -fill both; pack $t.seq1.panel -side top -fill both -expand true
        pack $t.seq2.head -side top -fill both; pack $t.seq2.panel -side top -fill both -expand true
        pack $t.program $t.seq1 $t.seq2 -side top -fill both
        
        #The Buttons Close and Help
        frame $t.buts -bd 2 -relief groove
        button $t.buts.ok -text "Run" -command "blastAlign_run $env(dbName)"
        button $t.buts.closeW -text "Close Window" -command "blastAlign_resClose; destroy $t"
        pack $t.buts.ok $t.buts.closeW -side left -expand 1 -anchor c
        pack $t.buts -side bottom -fill both
        
        wm geometry $t +[expr [winfo rootx .]+[winfo width .]/2]+[expr [winfo rooty .]+[winfo height .]/2]
        raise $t
    } else {
        ::errorPopUp "Error" "No database selected."
    }
}

proc blastAlign_run {$dbName} {
    global env
    global score_matrix_dna
    global gap_penalty   
    
    set seq1 [.localAlign.seq1.panel get -displaychars 1.0 end]
    set seq2 [.localAlign.seq2.panel get -displaychars 1.0 end]
    set program [.localAlign.program.cb get]
    if {$program eq "blastn"} {
        set dbtype nucl
    } else {
        set dbtype prot
    }
    if {[string index $seq1 0] ne ">"} {
        set seqH1 ">seq1"
        set seq1 [string toupper [string trim [regsub -all {\n} $seq1 ""]]]
    } else {
        set seq1 "\{[string trim [regsub {\n} $seq1 "\} \{"]]\}"
        set seqH1 "[lindex $seq1 0]"
        set seq1 [string toupper [string trim [regsub -all {\n} [lindex $seq1 1] ""]]]
    }
    if {[string index $seq2 0] ne ">"} {
        set seqH2 ">seq2"
        set seq2 [string toupper [string trim [regsub -all {\n} $seq2 ""]]]
    } else {
        set seq2 "\{[string trim [regsub {\n} $seq2 "\} \{"]]\}"
        set seqH2 "[lindex $seq2 0]"
        set seq2 [string toupper [string trim [regsub -all {\n} [lindex $seq2 1] ""]]]
    }
    set seq1Len [string length $seq1]
    set seq2Len [string length $seq2]  
    
    if {$seq1 ne ""} {
        if {$seq2 ne ""} {                        
            if {[catch {set ft [open [pwd]/blast_dbs/blastAlign.fasta w]} ]} {
            } else {
                puts $ft "$seqH2\n$seq2"
                close $ft
                if {$env(platform) == "windows"} {
                    set comm "makeblastdb.exe -parse_seqids -dbtype $dbtype -in [pwd]/blast_dbs/blastAlign.fasta -out [pwd]/blast_dbs/blastAlign"
                } elseif {$env(platform) == "unix"} {
                    set comm "makeblastdb -parse_seqids -dbtype $dbtype -in [pwd]/blast_dbs/blastAlign.fasta -out [pwd]/blast_dbs/blastAlign"
                }
                set err ""
                if {[catch {eval exec $comm} err]} {
                    ::errorPopUp module "Error: Format database failed.\n\n$err"
                }
                if {[catch {set ft [open [pwd]/blast_dbs/blastAlign.fasta w]} ]} {
                } else {
                    puts $ft "$seqH1\n$seq1"
                    close $ft
                }
            }
            if {$env(platform) == "windows"} {
                set cmd "$program.exe -task ${program}-short -db [pwd]/blast_dbs/blastAlign -query [pwd]/blast_dbs/blastAlign.fasta"
                catch {set str [eval exec $cmd]} str
            } elseif {$env(platform) == "unix"} {
                set cmd "$program -task ${program}-short -db [pwd]/blast_dbs/blastAlign -query [pwd]/blast_dbs/blastAlign.fasta"
                catch {set str [eval exec $cmd]} str
            }
            set blastAlignL ""
            set str [regsub -all {["]} $str ""]
            set str "{[regsub -all {\n} $str "\} \{"]}"
            if {[lindex [lindex $str 0] 2] eq "error:"} {
                ::errorPopUp "Error - BlastAlign" "Unexpected blast error."
            } else {
                set startQuery -1
                set startHit -1
                set matchLine 0
                set score ""
                set seqMatch ""
                set seqQuery ""
                set seqHit ""
                set mLStart ""
                foreach line $str {
                    if {[string index $line 0] eq ">"} {
                        if {$score ne ""} {
                            lappend blastArr(blastHitL) "$perfection $expect $score $startQuery $endQuery $startHit $endHit $strandQuery $strandHit [string toupper $seqQuery] [string toupper $seqHit] {$seqMatch}"
                            set score ""
                            set seqQuery ""
                            set seqMatch ""
                            set seqHit ""
                            set startQuery -1
                            set startHit -1
                            set mLStart ""
                        }
                    } elseif {[lindex $line 0] eq "Score"} {
                        if {$score ne ""} {
                            lappend blastAlignL "$perfection $expect $score $startQuery $endQuery $startHit $endHit $strandQuery $strandHit [string toupper $seqQuery] [string toupper $seqHit] {$seqMatch}"
                            set score ""
                            set seqQuery ""
                            set seqMatch ""
                            set seqHit ""
                            set startQuery -1
                            set startHit -1
                            set mLStart ""
                        }
                        set score [lindex $line 2]
                        set expect [lindex $line 7]
                        if {[string index $expect 0] eq "e"} {
                            set expect "1$expect"
                        }
                    } elseif {[lindex $line 0] eq "Identities"} {
                        set length [lindex [regsub -all {/} [lindex $line 2] " "] 1]
                        set perfection [regsub -all {\(} [regsub -all {\)} [regsub -all {,} [lindex $line 3] ""] ""] ""]
                    } elseif {[string range [lindex $line 0] 0 5] eq "Strand"} {
                        set line [regsub -all {[/=]} $line " "]
                        set strandQuery [lindex $line 1]
                        set strandHit [lindex $line 2]
                    } elseif {[lindex $line 0] eq "Query"} {
                        append seqQuery [lindex $line 2]
                        if {$startQuery eq -1} {
                            set startQuery [lindex $line 1]
                            set endQuery [lindex $line end]
                        } else {
                            set endQuery [lindex $line end]
                        }
                        set matchLine 1
                    } elseif {$matchLine} {
                        if {$mLStart eq ""} {
                            for {set i 0} {$i < [string length $line]} {incr i} {
                                if {[string index $line $i] ne " "} {
                                    set mLStart $i
                                    break
                                }
                            }
                        }
                        append seqMatch [string range $line $mLStart end]
                        set matchLine 0
                    } elseif {[lindex $line 0] eq "Sbjct"} {
                        append seqHit [lindex $line 2]
                        if {$startHit eq -1} {
                            set startHit [lindex $line 1]
                            set endHit [lindex $line end]
                        } else {
                            set endHit [lindex $line end]
                        }
                    }
                }
                if {$score ne ""} {
                    lappend blastAlignL "$perfection $expect $score $startQuery $endQuery $startHit $endHit $strandQuery $strandHit [string toupper $seqQuery] [string toupper $seqHit] {$seqMatch}"
                    unset seqQuery; unset seqMatch; unset seqHit
                }
            }
            
            set res [lindex $blastAlignL 0]
            
            set seqR1 ""; set seqR1_start ""; set seqR1_end ""; set seqR2 ""; set seqR2_start ""; set seqR2_end ""
            if {[lindex $res 7] eq "Plus" && [lindex $res 3] > 1} {
                set seqR1_start [string range $seq1 0 [expr [lindex $res 3] - 2]]
            } elseif {[lindex $res 7] eq "Minus" && [lindex $res 4] > 1} {
                set seqR1_start [string range $seq1 0 [expr [lindex $res 4] - 2]]
                set seqR1_start [string map {A T T A G C C G R Y Y R K M M K H D D H B V V B} $seqR1_start]
                for {set n [expr [string length $seqR1_start] - 1]} {$n >= 0} {set n [expr $n - 1]} {
                    append tmp [string index $seqR1_start $n]
                }
                set seqR1_start $tmp
                unset tmp
            }
            if {[lindex $res 8] eq "Plus" && [lindex $res 5] > 1} {
                set seqR2_start [string range $seq2 0 [expr [lindex $res 5] - 2]]
            } elseif {[lindex $res 8] eq "Minus" && [lindex $res 6] > 1} {
                set seqR2_start [string range $seq2 0 [expr [lindex $res 6] - 2]]
                set seqR2_start [string map {A T T A G C C G R Y Y R K M M K H D D H B V V B} $seqR2_start]
                for {set n [expr [string length $seqR2_start] - 1]} {$n >= 0} {set n [expr $n - 1]} {
                    append tmp [string index $seqR2_start $n]
                }
                set seqR2_start $tmp
                unset tmp
            }
            set diff [expr [string length $seqR1_start] - [string length $seqR2_start]]
            if {$diff > 10} {
                set seqR1 $seqR1_start
                set seqR2 $seqR1_start
            } elseif {$diff < -10} {
                set seqR1 $seqR2_start
                set seqR2 $seqR2_start
            } else {
                if {$diff > 0} {
                    for {set i 0} {$i < $diff} {incr i} {
                        append seqR2 " "
                    } 
                } else {
                    for {set i $diff} {$i < 0} {incr i} {
                        append seqR1 " "
                    }
                }
                append seqR1 $seqR1_start
                append seqR2 $seqR2_start
            }
            unset seqR1_start; unset seqR2_start; unset diff

            append seqR1 [lindex $res 9]
            append seqR2 [lindex $res 10]
            if {[lindex $res 7] eq "Plus" && [lindex $res 4] < [string length $seq1]} {
                append seqR1_end [string range $seq1 [lindex $res 4] end]
            } elseif {[lindex $res 7] eq "Minus" && [lindex $res 4] > 1} {
                set tmp [string range $seq1 0 [expr [lindex $res 4] - 2]]
                set tmp [string map {A T T A G C C G R Y Y R K M M K H D D H B V V B} $tmp]
                for {set n [expr [string length $tmp] - 1]} {$n >= 0} {set n [expr $n - 1]} {
                    append tmp2 [string index $tmp $n]
                }
                append seqR1_end $tmp2
                unset tmp; unset tmp2
            }
            if {[lindex $res 8] eq "Plus" && [lindex $res 6] < [string length $seq2]} {
                append seqR2_end [string range $seq2 [lindex $res 6] end]
            } elseif {[lindex $res 8] eq "Minus" && [lindex $res 6] > 1} {
                set tmp [string range $seq2 0 [expr [lindex $res 6] - 2]]
                set tmp [string map {A T T A G C C G R Y Y R K M M K H D D H B V V B} $tmp]
                for {set n [expr [string length $tmp] - 1]} {$n >= 0} {set n [expr $n - 1]} {
                    append tmp2 [string index $tmp $n]
                }
                append seqR2_end $tmp2
                unset tmp; unset tmp2
            }
            set diff [expr [string length $seqR1_end] - [string length $seqR2_end]]
            if {$diff > 10} {
                append seqR1 $seqR1_end
                append seqR2 $seqR1_end
            } elseif {$diff < -10} {
                append seqR1 $seqR2_end
                append seqR2 $seqR2_end
            } else {
                if {$diff > 0} {
                    for {set i 0} {$i < $diff} {incr i} {
                        append seqR2 " "
                    }
                } else {
                    for {set i $diff} {$i < 0} {incr i} {
                        append seqR1 " "
                    }
                }
                append seqR1 $seqR1_end
                append seqR2 $seqR2_end
            }
            unset seqR1_end; unset seqR2_end
            
            blastAlign_resWindow $seqR1 $seqR2 $seq1Len $seq2Len [lrange $res 0 8]
            
            catch {file delete [pwd]/blast_dbs/blastAlign.fasta}
        } else {
            ::errorPopUp "Error" "No sequence 2."
        }
    } else {
        ::errorPopUp "Error" "No sequence 1."
    }
}

proc blastAlign_resWindow {seqR1 seqR2 seq1Len seq2Len hit} {
    global resWinNum
    
    set t .localAlign_res$resWinNum
    toplevel $t
    wm title $t "Blast alignment"
    wm minsize $t 250 100
    wm resizable $t 1 0
    
    scrollbar $t.x -orient horizontal -command "move $resWinNum"
    frame $t.hit
    text $t.hit.panel -height 4 -background white
    frame $t.hit.export
    label $t.hit.export.label -text "New sequence:"
    button $t.hit.export.rb1 -text "Result 1" -command "catch {::database_nucleotide_new_window sequence_edit [regsub -all {\-} $seqR1 ""]}"
    button $t.hit.export.rb2 -text "Result 2" -command "catch {::database_nucleotide_new_window sequence_edit [regsub -all {\-} $seqR2 ""]}"
    frame $t.seq1
    label $t.seq1.label -text "Result 1 (length [string length [regsub -all {\-} $seqR1 ""]]): "
    text $t.seq1.panel -height 1 -background white -xscrollcommand "$t.x set" -wrap none
    frame $t.seq2
    label $t.seq2.label -text "Result 2 (length [string length [regsub -all {\-} $seqR2 ""]]): "
    text $t.seq2.panel -height 1 -background white -xscrollcommand "$t.x set" -wrap none
    
    pack $t.seq1.label -side left; pack $t.seq1.panel -side left -fill both -expand true
    pack $t.seq2.label -side left; pack $t.seq2.panel -side left -fill both -expand true
    pack $t.hit.export.label $t.hit.export.rb1 $t.hit.export.rb2 -side top -anchor c
    pack $t.hit.panel -side left; pack $t.hit.export -side right
    pack $t.seq1 $t.seq2 $t.x $t.hit -side top -fill both -expand true
    
    #The Buttons Close and Help
    frame $t.buts -bd 2 -relief groove
    button $t.buts.closeW -text "Close" -command "destroy $t"
    pack $t.buts.closeW -side left -expand 1 -anchor c
    pack $t.buts -side bottom -fill both
    
    wm geometry $t +[expr [winfo rootx .]+[winfo width .]/2]+[expr [winfo rooty .]+[winfo height .]/2]
    raise $t
    
    incr resWinNum
    $t.seq1.panel insert end $seqR1
    $t.seq2.panel insert end $seqR2
    
    set v2 $seq1Len
    for {set i 0} {$i < [expr 6 - [string length $seq1Len]]} {incr i} {
        append v2 " "
    }
    $t.hit.panel insert end "           \t        \tSeq1 length:\t$v2\tSeq2 length:\t$seq2Len\n"
    set v1 [lindex $hit 0]
    for {set i 0} {$i < [expr 8 - [string length [lindex $hit 0]]]} {incr i} {
        append v1 " "
    }
    set v2 [lindex $hit 3]
    for {set i 0} {$i < [expr 6 - [string length [lindex $hit 3]]]} {incr i} {
        append v2 " "
    }
    set v3 [lindex $hit 5]
    $t.hit.panel insert end "Perfection:\t$v1\tSeq1 start: \t$v2\tSeq2 start: \t$v3\n"
    set v1 [lindex $hit 1]
    for {set i 0} {$i < [expr 8 - [string length [lindex $hit 1]]]} {incr i} {
        append v1 " "
    }
    set v2 [lindex $hit 4]
    for {set i 0} {$i < [expr 6 - [string length [lindex $hit 4]]]} {incr i} {
        append v2 " "
    }
    set v3 [lindex $hit 6]
    $t.hit.panel insert end "e-value:   \t$v1\tSeq1 end:   \t$v2\tSeq2 end:   \t$v3\n"
    set v1 [lindex $hit 2]
    for {set i 0} {$i < [expr 8 - [string length [lindex $hit 2]]]} {incr i} {
        append v1 " "
    }
    set v2 [lindex $hit 7]
    for {set i 0} {$i < [expr 6 - [string length [lindex $hit 7]]]} {incr i} {
        append v2 " "
    }
    set v3 [lindex $hit 8]
    $t.hit.panel insert end "Score:     \t$v1\tSeq1 strand:\t$v2\tSeq2 strand:\t$v3\n"
    unset v1 v2 v3 seq1Len seq2Len
}

proc blastAlign_resClose {} {
    global resWinNum
    
    for {set i 0} {$i <= $resWinNum} {incr i} {
        destroy .localAlign_res$i
    }
}

proc move args {
    set winNum [lindex $args 0]
    set x 2
    if {[lindex $args 1] eq "moveto"} {
        for {set i 1} {$i <= $x} {incr i} {
            catch {
                .localAlign_res$winNum.seq$i.panel xview moveto [lindex $args 2]
            }
        }
    } elseif {[lindex $args 1] eq "scroll"} {
        for {set i 1} {$i <= $x} {incr i} {
            catch {
                .localAlign_res$winNum.seq$i.panel xview scroll [lindex $args 2] pages
            }
        }
    }
}
